@@ -5,15 +5,19 @@
 ;;;                       Copyright (c) 1996,1997                         ;;
 ;;;                        All Rights Reserved.                           ;;
 ;;;                                                                       ;;
-;;;  Permission to use, copy, modify, distribute this software and its    ;;
-;;;  documentation for research, educational and individual use only, is  ;;
-;;;  hereby granted without fee, subject to the following conditions:     ;;
+;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
+;;;  this software and its documentation without restriction, including   ;;
+;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
+;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
+;;;  permit persons to whom this work is furnished to do so, subject to   ;;
+;;;  the following conditions:                                            ;;
 ;;;   1. The code must retain the above copyright notice, this list of    ;;
 ;;;      conditions and the following disclaimer.                         ;;
 ;;;   2. Any modifications must be clearly marked as such.                ;;
 ;;;   3. Original authors' names are not deleted.                         ;;
-;;;  This software may not be used for commercial purposes without        ;;
-;;;  specific prior written permission from the authors.                  ;;
+;;;   4. The authors' names are not used to endorse or promote products   ;;
+;;;      derived from this software without specific prior written        ;;
+;;;      permission.                                                      ;;
 ;;;                                                                       ;;
 ;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
 ;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
@@ -29,30 +33,6 @@
 ;;;  General Festival Scheme specific functions
 ;;;  Including definitions of various standard variables.
 
-(defvar hush_startup nil
-  "hush_startup
-  If set to non-nil, the copyright banner is not displayed at start up.")
-
-(defvar readline_histsize 256
-  "readline_histsize
-  The number of line so be saved in the users history file when a 
-  Festival session ends.  The histfile is \".festival_history\" in the
-  users home directory.  Note this value is only checked when the 
-  command interpreter is started, hence this should be set in a user's
-  \".festivalrc\" or system init file.  Reseting it at the command
-  interpreter will have no effect.")
-
-(defvar ! nil
-  "!
-  In interactive mode, this variable's value is the return value of the
-  previously evaluated expression.")
-
-(defvar Parameter nil
-  "Parameter
-  An assoc-list of parameters and values for various parts of the speech
-  synthesis system.  This is used by the functions Parameter.set 
-  Parameter.def and Parameter.get as well as internal C++ functions.")
-
 ;; will be set automatically on start-up
 (defvar festival_version "unknown"
   "festival_version
@@ -64,40 +44,6 @@
  A list of major, minor and subminor version numbers of the current
  system.  e.g. (1 0 12).")
 
-(define (Parameter.set name val)
-"(Parameter.set NAME VAL)
-  Set parameter NAME to VAL (deleting any previous setting)"
-  (let ((pair (assoc name Parameter)))
-   (if pair
-       (set-car! (cdr pair) val)
-       (set! Parameter (cons (list name val) Parameter)))
-   val))
-
-(define (Parameter.def name val)
-"(Parameter.def NAME VAL)
-  Set parameter NAME to VAL if not already set"
-  (let ((pair (assoc name Parameter)))
-   (if pair
-       (car (cdr pair))
-       (and
-	(set! Parameter (cons (list name val) Parameter))
-	val))))
-
-(define (Parameter.get name)
-"(Parameter.get NAME)
-  Get parameter NAME's value (nil if unset)"
-  (let ((pair (assoc name Parameter)))
-    (if pair
-	(car (cdr pair))
-	nil)))
-(define (get_param name params default)
-  "(get_param name params default)
-Get named parameters in params returning default if its not present."
-  (let ((pair (assoc name params)))
-    (if pair
-	(car (cdr pair))
-	default)))
-
 (define (apply_method method utt)
 "(apply_method METHOD UTT)
 Apply the appropriate function to utt defined in parameter."
@@ -107,7 +53,7 @@ Apply the appropriate function to utt defined in parameter."
       nil)   ;; should be an error, but I'll let you off at present
      ((and (symbol? method_val) (symbol-bound? method_val))
       (apply (symbol-value method_val) (list utt)))
-     ((not (symbol? method_val))
+     ((member (typeof method_val) '(subr closure))
       (apply method_val (list utt)))
      (t      ;; again is probably an error
       nil))))
@@ -248,7 +194,7 @@ Return the second daughter of this item in this relation."
   (item.daughter2 (item.relation si relname)))
 
 (define (item.relation.daughtern si relname)
-  "(item.relation.daughter1 item relname)
+  "(item.relation.daughtern item relname)
 Return the final daughter of this item in this relation."
   (item.daughtern (item.relation si relname)))
 
@@ -270,9 +216,147 @@ Return the most previous item from this item in this relation."
      (set! n (item.prev n)))
     n))
 
+(define (item.leafs si)
+  "(item.relation.leafs item relname)
+Return a list of the leafs of this item in this relation."
+  (let ((ls nil)
+	(pl (item.first_leaf si))
+	(ll (item.next_leaf (item.last_leaf si))))
+    (while (and pl (not (equal? pl ll)))
+	   (set! ls (cons pl ls))
+	   (set! pl (item.next_leaf pl)))
+    (reverse ls)))
+
+(define (item.relation.leafs si relname)
+  "(item.relation.leafs item relname)
+Return a list of the leafs of this item in this relation."
+  (item.leafs (item.relation si relname)))
+
+(define (item.root s)
+  "(item.root s)
+Follow parent link until s has no parent."
+  (cond
+   ((item.parent s) 
+    (item.root (item.parent s)))
+   (t s)))
+
+(define (item.parent_to s relname)
+  "(item.parent_to s relname)
+Find the first ancestor of s in its current relation that is also in
+relname.  s is treated as an ancestor of itself so if s is in relname
+it is returned.  The returned value is in will be in relation relname
+or nil if there isn't one."
+  (cond
+   ((null s) s)
+   ((member_string relname (item.relations s)) 
+    (item.relation s relname))
+   (t (item.parent_to (item.parent s) relname))))
+
+(define (item.daughter1_to s relname)
+  "(item.daughter1_to s relname)
+Follow daughter1 links of s in its current relation until an item
+is found that is also in relname, is s is in relname it is returned.
+The return item is returned in relation relname, or nil if there is
+nothing in relname."
+  (cond
+   ((null s) s)
+   ((member_string relname (item.relations s)) (item.relation s relname))
+   (t (item.daughter1_to (item.daughter1 s) relname))))
+
+(define (item.daughtern_to s relname)
+  "(item.daughter1_to s relname)
+Follow daughtern links of s in its current relation until an item
+is found that is also in relname, is s is in relname it is returned.
+The return item is returned in relation relname, or nil if there is
+nothing in relname."
+  (cond
+   ((null s) s)
+   ((member_string relname (item.relations s)) (item.relation s relname))
+   (t (item.daughtern_to (item.daughtern s) relname))))
+
+(define (item.name s)
+"(item.name ITEM)
+  Returns the name of ITEM. [see Accessing an utterance]"
+  (item.feat s "name"))
+
+(define (utt.wave utt)
+  "(utt.wave UTT)
+Get waveform from wave (R:Wave.first.wave)."
+  (item.feat (utt.relation.first utt "Wave") "wave"))
+
+(define (utt.wave.rescale . args)
+ "(utt.wave.rescale UTT FACTOR NORMALIZE)
+Modify the gain of the waveform in UTT by GAIN.  If NORMALIZE is
+specified and non-nil the waveform is maximized first."
+  (wave.rescale (utt.wave (nth 0 args)) (nth 1 args) (nth 2 args))
+  (nth 0 args))
+
+(define (utt.wave.resample utt rate)
+  "(utt.wave.resample UTT RATE)\
+Resample waveform in UTT to RATE (if it is already at that rate it remains
+unchanged)."
+  (wave.resample (utt.wave utt) rate)
+  utt)
+
+(define (utt.import.wave . args)
+  "(utt.import.wave UTT FILENAME APPEND)
+Load waveform in FILENAME into UTT in R:Wave.first.wave.  If APPEND
+is specified and non-nil append this to the current waveform."
+  (let ((utt (nth 0 args))
+	(filename (nth 1 args))
+	(append (nth 2 args)))
+    (if (and append (member 'Wave (utt.relationnames utt)))
+	(wave.append (utt.wave utt) (wave.load filename))
+	(begin
+	  (utt.relation.create utt 'Wave)
+	  (item.set_feat
+	   (utt.relation.append utt 'Wave)
+	   "wave"
+	   (wave.load filename))))
+    utt))
+
+(define (utt.save.wave . args)
+  "(utt.save.wave UTT FILENAME FILETYPE)
+Save waveform in UTT in FILENAME with FILETYPE (if specified) or
+using global parameter Wavefiletype."
+  (wave.save 
+   (utt.wave (nth 0 args))
+   (nth 1 args)
+   (nth 2 args))
+  (nth 0 args))
+
+(define (utt.play utt)
+  "(utt.play UTT)
+Play waveform in utt by current audio method."
+  (wave.play (utt.wave utt))
+  utt)
+
+(define (utt.save.track utt filename relation feature)
+  "(utt.save.track utt filename relation feature)
+DEPRICATED use trace.save instead."
+  (format stderr "utt.save.track: DEPRICATED use track.save instead\n")
+  (track.save 
+   (item.feat
+    (utt.relation.first utt relation)
+    feature)
+   filename)
+  utt)
+
+(define (utt.import.track utt filename relation fname)
+  "(utt.import.track UTT FILENAME RELATION FEATURE_NAME)
+Load track in FILENAME into UTT in R:RELATION.first.FEATURE_NAME.
+Deletes RELATION if it already exists. (you maybe want to use track.load
+directly rather than this legacy function."
+  (utt.relation.create utt relation)
+  (item.set_feat
+   (utt.relation.append utt relation)
+   fname
+   (track.load filename))
+  utt)
+
 (define (wagon_predict item tree)
-"(wagon_predict UTT ITEM TREE)
-Predict with given CART tree, UTT nad ITEM and return the prediction
+"(wagon_predict ITEM TREE)
+Predict with given ITEM and CART tree and return the prediction
 (the last item) rather than whole probability distribution."
  (car (last (wagon item tree))))
 
@@ -381,7 +465,7 @@ Getting Help
   (manual nil)      displays manual in local netscape
   C-c               return to top level
   C-d or (quit)     Exit Festival
-(If compiled with readline)
+(If compiled with editline)
   M-h               desplays help on current symbol  
   M-s               speaks help on current symbol  
   M-m               displays relevant manula page in local netscape
@@ -398,7 +482,7 @@ Doing stuff
   (tts FILENAME nil)  Say contexts of file, FILENAME should be 
                       surrounded by double quotes
   (voice_rab_diphone) Select voice (Britsh Male)
-  (voice_kd_diphone)  Select voice (American Male)
+  (voice_ked_diphone) Select voice (American Male)
 ")
 
 (define (festival_warranty)
@@ -411,18 +495,22 @@ Doing stuff
 "
                 Centre for Speech Technology Research                  
                      University of Edinburgh, UK                       
-                       Copyright (c) 1996-1998                         
+                       Copyright (c) 1996-2002
                         All Rights Reserved.                           
                                                                        
-  Permission to use, copy, modify, distribute this software and its            
-  documentation for research, educational and individual use only, is  
-  hereby granted without fee, subject to the following conditions:     
+  Permission is hereby granted, free of charge, to use and distribute  
+  this software and its documentation without restriction, including   
+  without limitation the rights to use, copy, modify, merge, publish,  
+  distribute, sublicense, and/or sell copies of this work, and to      
+  permit persons to whom this work is furnished to do so, subject to   
+  the following conditions:                                            
    1. The code must retain the above copyright notice, this list of    
       conditions and the following disclaimer.                         
    2. Any modifications must be clearly marked as such.                
    3. Original authors' names are not deleted.                         
-  This software may not be used for commercial purposes without        
-  specific prior written permission from the authors.                  
+   4. The authors' names are not used to endorse or promote products   
+      derived from this software without specific prior written        
+      permission.                                                      
                                   
   THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        
   DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      
@@ -512,4 +600,16 @@ Compile all the scheme files in the library directory."
      "tobi.scm" "f2bf0lr.scm"))
   t)
 
+(define (register_unisyn_features ddd)
+  (format t "********************\n")
+  (format t "\n")
+  (format t "The Scheme `register_unisyn_features' is no longer required\n")
+  (format t "its function is absorbed into the initialization process\n")
+  (format t "delete you call to this function now as this will cause\n")
+  (format t "an error in the future\n")
+  (format t "\n")
+  (system "sleep 3")
+  (format t "********************\n")
+)  
+
 (provide 'festival)
