@@ -5,15 +5,19 @@
 ;;;                         Copyright (c) 1998                            ;;
 ;;;                        All Rights Reserved.                           ;;
 ;;;                                                                       ;;
-;;;  Permission to use, copy, modify, distribute this software and its    ;;
-;;;  documentation for research, educational and individual use only, is  ;;
-;;;  hereby granted without fee, subject to the following conditions:     ;;
+;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
+;;;  this software and its documentation without restriction, including   ;;
+;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
+;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
+;;;  permit persons to whom this work is furnished to do so, subject to   ;;
+;;;  the following conditions:                                            ;;
 ;;;   1. The code must retain the above copyright notice, this list of    ;;
 ;;;      conditions and the following disclaimer.                         ;;
 ;;;   2. Any modifications must be clearly marked as such.                ;;
 ;;;   3. Original authors' names are not deleted.                         ;;
-;;;  This software may not be used for commercial purposes without        ;;
-;;;  specific prior written permission from the authors.                  ;;
+;;;   4. The authors' names are not used to endorse or promote products   ;;
+;;;      derived from this software without specific prior written        ;;
+;;;      permission.                                                      ;;
 ;;;                                                                       ;;
 ;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
 ;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
@@ -90,7 +94,7 @@ Find all feasible alignments."
 			 (lambda (p)
 			   (cons (cons (car phones) (car letters)) p))
 			 (find-aligns (cdr phones) (cdr letters))))))
-      ;; Hmm, change this to always chek doubles
+      ;; Hmm, change this to always check doubles
       (if (valid-pair-e (car phones) (car (cdr phones)) (car letters))
 	  (set! r
 		(append r
@@ -258,7 +262,7 @@ Aligns all possible ways for these strings."
 		     lets))
 	   (if (not bp)
 	       (format t "failed: %l\n" lets)
-	       (save_info bp ofd))
+	       (save_info (car (cdr entry)) bp ofd))
 	   (set! c (+ 1 c)))
     (fclose fd)
     (fclose ofd)))
@@ -266,8 +270,8 @@ Aligns all possible ways for these strings."
 (define (enworden lets)
   (cons '# (reverse (cons '# (reverse lets)))))
 
-(define (save_info bp ofd)
-  "(make_word_utt phones letters c)
+(define (save_info pos bp ofd)
+  "(save_info pos bp ofd)
 Cut out one expensive step and 50M of diskspace and just save it
 in a simpler format."
   (format ofd "( ")
@@ -276,6 +280,7 @@ in a simpler format."
      (if (not (string-equal "#" (cdr l)))
 	 (format ofd "%s" (cdr l))))
    bp)
+  (format ofd " %s" pos)
   (mapcar
    (lambda (l)
      (if (not (string-equal "#" (car l)))
@@ -313,16 +318,17 @@ file contain predicted phone, and letter with 3 preceding and
   (let ((fd (fopen alignfile "r"))
 	(ofd (fopen featfile "w"))
 	(entry)
-	(pn))
+	(pn)
+	(sylpos 1))
     (while (not (equal? (set! entry (readfp fd)) (eof-val)))
 	   (set! lets (append '(0 0 0 0 #) (symbolexplode (car entry))
 			      '(# 0 0 0 0)))
-	   (set! phones (cdr entry))
+	   (set! phones (cdr (cdr entry)))
 	   (set! pn 5)
 	   (mapcar
 	    (lambda (p)
 	      (format ofd
-		      "%s  %s %s %s %s  %s  %s %s %s %s\n"
+		      "%s  %s %s %s %s  %s  %s %s %s %s  %s\n"
 		      p
 		      (nth (- pn 4) lets)
 		      (nth (- pn 3) lets)
@@ -332,7 +338,12 @@ file contain predicted phone, and letter with 3 preceding and
 		      (nth (+ pn 1) lets)
 		      (nth (+ pn 2) lets)
 		      (nth (+ pn 3) lets)
-		      (nth (+ pn 4) lets))
+		      (nth (+ pn 4) lets)
+		      (car (cdr entry)) ;; pos
+		      ;; sylpos
+		      ;; numsyls
+		      ;; num2end
+		      )
 	      (set! pn (+ 1 pn)))
 	    phones))
     (fclose fd)
@@ -362,16 +373,6 @@ named by name, in filename."
     (fclose fd))
 )
 
-(define (lts_function_split word feats lts_rules)
-  "(lts_function_split word feats lts_rules)
-Predict using split lts/stress rules (assumes that OALD is the current
-lexicon."
-  (require 'lts)
-  (set! phones (lts_predict word lts_rules))
-  (set! phones (add_lex_stress word feats phones english_stress_tree))
-  (enworden 
-   (mapcar intern phones)))
-
 (define (lts_testset file cartmodels)
   "(lts_testset file cartmodels)
 Test an aligned lexicon file against a set of cart trees.  Prints out
@@ -382,69 +383,39 @@ the structure as saved by merge_models."
 	(entry)
 	(wordcount 0)
 	(correctwords 0)
-	(correctwords_ns 0)
 	(phonecount 0)
-	(correctphones 0)
-	(correctphones_ns 0))
-    (set! lts_cap 'C)
+	(correctphones 0))
     (while (not (equal? (set! entry (readfp fd)) (eof-val)))
 	   (let ((letters (enworden (symbolexplode (car entry))))
 		 (phones (enworden (cdr (cdr entry))))
 		 (pphones))
 	     (set! wordcount (+ 1 wordcount))
-	     (set! lts_pos (car (cdr entry)))
-	     (if (string-equal (car entry) "abased")
-		 (set! lts_cap 'U))
-	     (set! pphones (gen_cartlts letters cartmodels))
-;	     (set! pphones (lts_function_split (car entry) 
-;					       (car (cdr entry))
-;					       cartmodels))
+	     (set! pphones (gen_cartlts letters (car (cdr entry)) cartmodels))
+;	     (set! pphones (gen_vilts letters (car (cdr entry))
+;				      cartmodels wfstname))
 	     (if (equal? (ph-normalize pphones) (ph-normalize phones))
 		 (set! correctwords (+ 1 correctwords))
-		 (or t
+		 (or nil
 		     (format t "failed %s %l %l\n" (car entry) phones pphones)))
-	     (if (equal? (unstress (ph-normalize pphones))
-			 (unstress (ph-normalize phones)))
-		 (set! correctwords_ns (+ 1 correctwords_ns)))
-	     (if (and (not (equal? (ph-normalize pphones) (ph-normalize phones)))
-		      (equal? (unstress (ph-normalize pphones))
-			      (unstress (ph-normalize phones))))
-		 (format t "stress mismatch %s %l %l\n"
-			 (car entry) phones pphones))
 	     (count_correct_letters   ;; exclude #, cause they're always right
 	      (cdr letters)
 	      (cdr phones)
-	      (cdr pphones)
-	      'correct_letter_table)
-	     (count_correct_letters   ;; exclude #, cause they're always right
-	      (cdr letters)
-	      (unstress (cdr phones))
-	      (unstress (cdr pphones))
-	      'correct_letter_table_ns)
+	      (cdr pphones))
 	     (set! phonecount (+ (length (cdr (cdr letters))) phonecount))
 	     ))
     (fclose fd)
     (mapcar
      (lambda (linfo)
-       (format t "%s %d correct %d (%2.2f) %d nodes\n"
+       (format t "%s %d correct %d (%2.2f)\n"
 	       (car linfo) (car (cdr linfo))
 	       (car (cdr (cdr linfo)))
-	       (/ (* (car (cdr (cdr linfo))) 100) (car (cdr linfo)))
-	       (cart_tree_node_count (car (cdr (assoc (car linfo) cartmodels)))))
+	       (/ (* (car (cdr (cdr linfo))) 100) (car (cdr linfo))))
        (set! correctphones (+ correctphones (car (cdr (cdr linfo))))))
      correct_letter_table)
-    (mapcar
-     (lambda (linfo)
-       (set! correctphones_ns (+ correctphones_ns (car (cdr (cdr linfo))))))
-     correct_letter_table_ns)
     (format t "phones %d correct %d (%2.2f)\n"
 	    phonecount correctphones (/ (* correctphones 100) phonecount))
-    (format t "phones %d correct %d (%2.2f) ignoring stress\n"
-	    phonecount correctphones_ns (/ (* correctphones_ns 100) phonecount))
     (format t "words %d correct %d (%2.2f)\n"
 	    wordcount correctwords (/ (* correctwords 100) wordcount))
-    (format t "words %d correct %d (%2.2f) ignoring stress\n"
-	    wordcount correctwords_ns (/ (* correctwords_ns 100) wordcount))
     (format t "tree model has %d nodes\n"
 	    (apply + (mapcar (lambda (a) (cart_tree_node_count (car (cdr a))))
 			     cartmodels)))
@@ -471,15 +442,8 @@ phone predictions.  This list will be extended if there are more letters
 in your alphabet, though it doesn't take a fairly western european
 view of the alphabet,  but you can change this yourself is necessary.")
 
-(defvar correct_letter_table_ns
-  (mapcar
-   (lambda (l) (list l 0 0))
-   '(a b c d e f g h i j k l m n o p q r s t u v w x y z))
-  "correct_letter_table_ns
-Same as corect_letter_table but ignoring stress markers.")
-
-(define (count_correct_letters lets phs pphs table)
- "(count_correct_letters lets phs pphs table)
+(define (count_correct_letters lets phs pphs)
+ "(count_correct_letters lets phs pphs)
 Count which letters have the correct phone prediction.  Cummulate this
 is a per letter table."
  (cond
@@ -489,17 +453,15 @@ is a per letter table."
   ((and (null (cdr lets)) (null (cdr phs)) (null (cdr pphs)))
    nil)  ;; omit final #
   (t
-   (let ((letinfo (assoc (car lets) (symbol-value table))))
+   (let ((letinfo (assoc (car lets) correct_letter_table)))
      (if (not letinfo)
-	 (begin
-	   (format t "new let %s\n" (car lets))
-	 (set table
-	       (append (symbol-value table)
-		       (set! letinfo (list (list (car lets) 0 0)))))))
+	 (set! correct_letter_table
+	       (append correct_letter_table
+		       (set! letinfo (list (list (car lets) 0 0))))))
      (set-car! (cdr letinfo) (+ 1 (car (cdr letinfo)))) ;; total
-     (if (string-equal (car phs) (car pphs))                  ;; correct 
+     (if (equal? (car phs) (car pphs))                  ;; correct 
 	 (set-car! (cdr (cdr letinfo)) (+ 1 (car (cdr (cdr letinfo))))))
-     (count_correct_letters (cdr lets) (cdr phs) (cdr pphs) table)))))
+     (count_correct_letters (cdr lets) (cdr phs) (cdr pphs))))))
 
 (define (ph-normalize ph)
   (cond
@@ -515,21 +477,9 @@ is a per letter table."
    (t
     (cons (car ph) (ph-normalize (cdr ph))))))
 
-(define (unstress ph)
-  (cond
-   ((null ph) nil)
-   ((string-matches (car ph) ".*1$")
-    (cons
-     (string-before (car ph) "1")
-     (unstress (cdr ph)))) 
-  (t
-    (cons
-     (car ph)
-     (unstress (cdr ph))))))
-
 (define (make_let_utt_p letters pos)
 "(make_let_utt letters)
-Build an utterances from these letters."
+Build an utterances from th4ese letters."
   (let ((utt (Utterance Text "")))
     (utt.relation.create utt 'LTS)
     (utt.relation.create utt 'LETTER)
@@ -539,27 +489,48 @@ Build an utterances from these letters."
      (lambda (l)
        (let ((lsi (utt.relation.append utt 'LETTER)))
 	 (item.set_name lsi l)
-	 (item.set_feat lsi "cap" lts_cap)
-	 (item.set_feat lsi "end_let" (car (cdr (reverse letters))))
 	 (item.set_feat lsi "pos" pos)))
      letters)
     utt))
 
-(define (gen_cartlts letters cartmodels)
+(define (gen_vilts letters pos cartmodels ngram)
+  "(get_vilts letters pos cartmodels ngram)
+Use cart plus ngrams in viterbi search."
+  (require 'lts)
+  (let ((utt (make_let_utt_p letters pos)))
+    (set! gen_vit_params
+	  (list
+	   (list 'Relation "LETTER")
+	   (list 'return_feat "phone")
+	   (list 'p_word "#")
+	   (list 'pp_word "0")
+	   (list 'wfstname ngram)
+	   (list 'cand_function 'lts_cand_function)))
+    (Gen_Viterbi utt)
+    (mapcar 
+      (lambda (lsi)
+	(intern (item.feat lsi "phone")))
+      (utt.relation.items utt 'LETTER))))
+
+(define (gen_cartlts letters pos cartmodels)
   "(get_cartlts letters cartmodels)
 Generate the full list of predicted phones, including
 epsilon and unexpanded multi-phones."
   (require 'lts)
-  (let ((utt (make_let_utt_p letters lts_pos)))
+  (let ((utt (make_let_utt_p letters pos)))
     (enworden
      (mapcar
       (lambda (lsi)
-	(let ((tree (car (cdr (assoc_string (item.name lsi) cartmodels)))))
+	(let ((tree (car (cdr (assoc_string (item.name lsi) cartmodels))))
+	      (p))
 	  (if (not tree)
 	      (begin
 		(format t "failed to find tree for %s\n" (item.name lsi))
 		nil)
-	      (wagon_predict lsi tree))))
+	      (begin
+		(set! p (wagon_predict lsi tree))
+		(item.set_feat lsi "val" p)
+		p))))
       (reverse (cdr (reverse (cdr (utt.relation.items utt 'LETTER)))))))))
 
 (define (reduce_lexicon entryfile exceptionfile lts_function)
