@@ -5,13 +5,19 @@
 ;;;                         Copyright (c) 1998                            ;;
 ;;;                        All Rights Reserved.                           ;;
 ;;;                                                                       ;;
-;;;  Permission to use, copy, modify,  and licence this software and its  ;;
-;;;  documentation for any purpose, is hereby granted without fee,        ;;
-;;;  subject to the following conditions:                                 ;;
+;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
+;;;  this software and its documentation without restriction, including   ;;
+;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
+;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
+;;;  permit persons to whom this work is furnished to do so, subject to   ;;
+;;;  the following conditions:                                            ;;
 ;;;   1. The code must retain the above copyright notice, this list of    ;;
 ;;;      conditions and the following disclaimer.                         ;;
 ;;;   2. Any modifications must be clearly marked as such.                ;;
 ;;;   3. Original authors' names are not deleted.                         ;;
+;;;   4. The authors' names are not used to endorse or promote products   ;;
+;;;      derived from this software without specific prior written        ;;
+;;;      permission.                                                      ;;
 ;;;                                                                       ;;
 ;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
 ;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
@@ -39,6 +45,23 @@
 ;;       (cons "\\.sable$" 'sable)
 ;;       auto-text-mode-alist))
 
+ ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+ ;;                                                                       ;;
+ ;; Remember where to find these two XML entities.                        ;;
+ ;;                                                                       ;;
+ ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+
+(xml_register_id "-//SABLE//DTD SABLE speech mark up//EN"
+		(path-append libdir "Sable.v0_2.dtd")
+		)
+
+(xml_register_id "-//SABLE//ENTITIES Added Latin 1 for SABLE//EN"
+		 (path-append libdir  "sable-latin.ent")
+		 )
+
+;; (print (xml_registered_ids))
+
 (defvar SABLE_RXDOUBLE "-?\\(\\([0-9]+\\.[0-9]*\\)\\|\\([0-9]+\\)\\|\\(\\.[0-9]+\\)\\)\\([eE][---+]?[0-9]+\\)?")
 
 (defvar sable_pitch_base_map
@@ -98,10 +121,22 @@ SABLE mode token specific analysis."
    ((or sable_omitted_mode sable_unsupported_language)
     ;; don't say anything (whole utterance)
     nil)
+   ((string-equal "1" (item.feat token "done_sable_sub"))
+    ;; to catch recursive calls this when splitting up sub expressions
+    (sable_previous_token_to_words token name))
    ((and (not (string-equal "0" (item.feat token "sable_sub")))
 	 (string-equal "0" (item.feat token "p.sable_sub")))
-    ;; this wont catch the case where the PRON doesn't contain any words
-    (read-from-string (item.feat token "sable_sub")))
+    (let (words (sub (item.feat token "sable_sub")))
+      (item.set_feat token "done_sable_sub" "1")
+      (set! words 
+	    (apply append
+		   (mapcar
+		    (lambda (w)
+		      (set! www (sable_previous_token_to_words token w))
+		      www)
+		    (read-from-string sub))))
+      (item.set_feat token "done_sable_sub" "0")
+      words))
    ((string-equal "1" (item.feat token "sable_ignore"))
     ;; don't say anything (individual word)
     nil)
@@ -124,7 +159,7 @@ SABLE mode token specific analysis."
       subwords))
    ((not (string-equal "0" (item.feat token "token_pos")))
     ;; bypass the prediction stage, if English
-    (if (member_string (car (cdr (assoc 'Language Parameter)))
+    (if (member_string (Parameter.get 'Language)
 		       '(britishenglish americanenglish))
 	(builtin_english_token_to_words token name)
 	(sable_previous_token_to_words token name)))
@@ -198,14 +233,20 @@ SABLE mode token specific analysis."
     ((equal? (car (xxml_attval "NAME" ATTLIST)) 'male3)
      (set! sable_current_speaker 'voice_ked_diphone)
      (voice_ked_diphone))
-    (t
-     (print "SABLE: selecting unknown voice")
-     (set! sable_current_speaker voice_rab_diphone)
-     (voice_rab_diphone)))
+    ((equal? (car (xxml_attval "NAME" ATTLIST)) 'female1)
+     (set! sable_current_speaker 'voice_us1_mbrola)
+     (voice_us1_mbrola))
+    ((equal? (car (xxml_attval "NAME" ATTLIST)) 'male4)
+     (set! sable_current_speaker 'voice_us2_mbrola)
+     (voice_us2_mbrola))
+   (t
+      (set! sable_current_speaker (intern (string-append "voice_" (car (xxml_attval "NAME" ATTLIST)))))
+      (eval (list sable_current_speaker))))
     (sable_setup_voice_params)
    nil)
   (")SPEAKER" (ATTLIST UTT)
    (xxml_synth UTT)
+   (set! sable_utt UTT)
    (set! sable_current_speaker (car sable_speaker_stack))
    (set! sable_speaker_stack (cdr sable_speaker_stack))
    (eval (list sable_current_speaker))
@@ -215,14 +256,16 @@ SABLE mode token specific analysis."
    ;; Status: probably complete
    ;; may cause an utterance break
    (let ((level (upcase (car (xxml_attval "LEVEL" ATTLIST)))))
-     (if (string-equal "LARGE" level)
-	 (begin
-	   (xxml_synth UTT)
-	   nil)
-	 (let ((last_token (utt.relation.last UTT'Token)))
-	   (if last_token
-	       (item.set_feat last_token "pbreak" "B"))
-	   UTT))))
+     (cond
+      ((null UTT) nil)
+      ((string-equal "LARGE" level)
+       (xxml_synth UTT)
+       nil)
+      (t
+       (let ((last_token (utt.relation.last UTT'Token)))
+	 (if last_token
+	     (item.set_feat last_token "pbreak" "B"))
+	 UTT)))))
   ("(DIV" (ATLIST UTT)
    ;; Status: probably complete
    (xxml_synth UTT)
