@@ -5,15 +5,19 @@
 ;;;                       Copyright (c) 1996,1997                         ;;
 ;;;                        All Rights Reserved.                           ;;
 ;;;                                                                       ;;
-;;;  Permission to use, copy, modify, distribute this software and its    ;;
-;;;  documentation for research, educational and individual use only, is  ;;
-;;;  hereby granted without fee, subject to the following conditions:     ;;
+;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
+;;;  this software and its documentation without restriction, including   ;;
+;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
+;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
+;;;  permit persons to whom this work is furnished to do so, subject to   ;;
+;;;  the following conditions:                                            ;;
 ;;;   1. The code must retain the above copyright notice, this list of    ;;
 ;;;      conditions and the following disclaimer.                         ;;
 ;;;   2. Any modifications must be clearly marked as such.                ;;
 ;;;   3. Original authors' names are not deleted.                         ;;
-;;;  This software may not be used for commercial purposes without        ;;
-;;;  specific prior written permission from the authors.                  ;;
+;;;   4. The authors' names are not used to endorse or promote products   ;;
+;;;      derived from this software without specific prior written        ;;
+;;;      permission.                                                      ;;
 ;;;                                                                       ;;
 ;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
 ;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
@@ -38,11 +42,11 @@ to choose the appropriate token to word module."
 	(language (Parameter.get 'Language)))
     (cond
      (rval rval)  ;; newer style
-     ((or (eq? 'britishenglish language)
-	  (eq? 'english language)
-	  (eq? 'americanenglish language))
+     ((or (string-equal "britishenglish" language)
+	  (string-equal "english" language)
+	  (string-equal "americanenglish" language))
       (Token_English utt))
-     ((eq? 'welsh language)
+     ((string-equal "welsh" language)
       (Token_Welsh utt))
      (t
       (Token_Any utt)))))
@@ -141,6 +145,7 @@ rules are not used unless explicitly called. [see Token to word rules]"
 	    nil
 	    (list '((name "'s")(pos nnp)))))))))
   ((and (string-matches name "No")
+	(item.next token)
         (string-matches (item.feat token "n.name")
 			"[0-9]+"))
    (list
@@ -569,10 +574,14 @@ Preceeding puctuation symbol found before token in original string/file.")
      ((p.name in (was were had been having has is are))
       ((led))
       ((liid))))
+    ("read"
+     ((p.name in (to))
+      ((riid))
+      ((red))))
     ))
 
 (defvar english_homographs
-  '("lead")
+  '("lead" "read")
   "english_homographs
 A list of tokens that are dealt with by a homograph disambiguation tree
 in english_token_pos_cart_trees.")
