@@ -5,15 +5,19 @@
 /*                       Copyright (c) 1996,1997                         */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -44,54 +48,21 @@ static void ps_add_def(const EST_String &name, PhoneSet *ps);
 static LISP lisp_select_phoneset(LISP phoneset);
 
 static LISP phone_set_list = NULL;
-static int tc_phone = -1;
-static int tc_phoneset = -1;
 static PhoneSet *current_phoneset = NULL;
 
 static const EST_String f_cvox("cvox");
 static const EST_String f_vc("vc");
 static const EST_String f_ctype("ctype");
 
-static class Phone *get_c_phone(LISP x)
-{
-    if (TYPEP(x,tc_phone))
-	return (class Phone *)USERVAL(x);
-    else
-	err("wta to get_c_phone",x);
-
-    return NULL;  // err doesn't return but compilers don't know that
-}
-
-static LISP siod_make_phone(Phone *s)
-{
-    if (s==0)
-	return NIL;
-    else
-	return siod_make_typed_cell(tc_phone,s);
-}
-
-static class PhoneSet *get_c_phoneset(LISP x)
-{
-    if (TYPEP(x,tc_phoneset))
-	return (class PhoneSet *)USERVAL(x);
-    else
-	err("wta to get_c_phoneset",x);
-
-    return NULL;  // err doesn't return but compilers don't know that
-}
-
-static LISP siod_make_phoneset(PhoneSet *s)
-{
-    if (s==0)
-	return NIL;
-    else
-	return siod_make_typed_cell(tc_phoneset,s);
-}
+VAL_REGISTER_CLASS(phone,Phone)
+SIOD_REGISTER_CLASS(phone,Phone)
+VAL_REGISTER_CLASS(phoneset,PhoneSet)
+SIOD_REGISTER_CLASS(phoneset,PhoneSet)
 
 int Phone::match_features(Phone *foreign)
 {
     // Try to match the features of the foreign phone with this one
-    EST_TBI *f;
+    EST_Litem *f;
 
     for (f=features.list.head(); f != 0; f=next(f))
     {	
@@ -111,14 +82,14 @@ PhoneSet::~PhoneSet()
     gc_unprotect(&phones);
 }
 
-Phone * PhoneSet::member(const EST_String &phone) const
+Phone * PhoneSet::member(const EST_String &ph) const
 {
-    LISP p = siod_assoc_str(phone,phones);
+    LISP p = siod_assoc_str(ph,phones);
     if (p != 0)
-	return get_c_phone(car(cdr(p)));
+	return phone(car(cdr(p)));
     else 
     {
-	cerr << "Phone \"" << phone << "\" not member of PhoneSet \"" << 
+	cerr << "Phone \"" << ph << "\" not member of PhoneSet \"" << 
 	    psetname << "\"" << endl;
 	return 0;
     }
@@ -154,7 +125,7 @@ int PhoneSet::add_phone(Phone *phone)
     if (lpair == NIL)
     {
 	phones = cons(make_param_lisp(phone->phone_name(),
-				      siod_make_phone(phone)),
+				      siod(phone)),
 		      phones);
 	return TRUE;
     }
@@ -205,8 +176,8 @@ Phone *PhoneSet::find_matched_phone(Phone *foreign)
 
     for (p=phones; p != NIL; p=cdr(p))
     {
-	if (get_c_phone(car(cdr(car(p))))->match_features(foreign))
-	    return get_c_phone(car(cdr(car(p))));
+	if (phone(car(cdr(car(p))))->match_features(foreign))
+	    return phone(car(cdr(car(p))));
     }
 
     // could try harder 
@@ -236,6 +207,16 @@ void PhoneSet::set_map(LISP m)
     map=m;
 }
 
+PhoneSet &PhoneSet::operator = (const PhoneSet &a)
+{
+    psetname = a.psetname;
+    silences = a.silences;
+    map = a.map;
+    feature_defs = a.feature_defs;
+    phones = a.phones;
+    return *this;
+}
+
 LISP make_phoneset(LISP args,LISP env)
 {
     // define a new phoneme set 
@@ -317,14 +298,14 @@ PhoneSet *phoneset_name_to_set(const EST_String &name)
 	festival_error();
     }
     
-    return get_c_phoneset(car(cdr(lpair)));
+    return phoneset(car(cdr(lpair)));
 
 }
 
-static LISP lisp_select_phoneset(LISP phoneset)
+static LISP lisp_select_phoneset(LISP pset)
 {
     // Select named phoneset and make it current
-    EST_String name = get_c_string(phoneset);
+    EST_String name = get_c_string(pset);
     LISP lpair;
 
     lpair = siod_assoc_str(name,phone_set_list);
@@ -335,33 +316,31 @@ static LISP lisp_select_phoneset(LISP phoneset)
 	festival_error();
     }
     else
-	current_phoneset = get_c_phoneset(car(cdr(lpair)));
-
-    return phoneset;
+	current_phoneset = phoneset(car(cdr(lpair)));
 
+    return pset;
 }
 
 static void ps_add_def(const EST_String &name, PhoneSet *ps)
 {
     //  Add phoneset to list of phonesets
     LISP lpair;
-    PhoneSet *old_ps;
+
+    if (phone_set_list == NIL)
+	gc_protect(&phone_set_list);
 
     lpair = siod_assoc_str(name,phone_set_list);
     
     if (lpair == NIL)
     {
 	phone_set_list = cons(cons(rintern(name),
-				   cons(siod_make_phoneset(ps),NIL)),
+				   cons(siod(ps),NIL)),
 			      phone_set_list);
-	gc_protect(&phone_set_list);
     }
     else
     {
 	cwarn << "Phoneset \"" << name << "\" redefined" << endl;
-	old_ps = get_c_phoneset(car(cdr(lpair)));
-	delete old_ps;
-	setcar(cdr(lpair),siod_make_phoneset(ps));
+	setcar(cdr(lpair),siod(ps));
     }
 
     return;
@@ -380,8 +359,8 @@ static void check_phoneset(void)
 
 static EST_Val ff_ph_feature(EST_Item *s,const EST_String &name)
 {
-    // This function is called for all phone feature 
-    // It looks at the name ff_feature_name to find out the
+    // This function is called for all phone features.
+    // It looks at the name to find out the
     // the actual name used to call this feature and removed the
     // ph_ prefix and uses the remainder as the phone feature name
     Phone *phone_def;
@@ -426,7 +405,7 @@ static PhoneSet *find_phoneset(EST_String name)
 	cerr << "Phoneset \"" << name << "\" not defined" << endl;
 	festival_error();
     }
-    return get_c_phoneset(car(cdr(lpair)));
+    return phoneset(car(cdr(lpair)));
 
 }
 
@@ -664,9 +643,6 @@ void festival_Phone_init(void)
 {
     // define Lisp accessor functions 
 
-    tc_phone = siod_register_user_type("Phone");
-    tc_phoneset = siod_register_user_type("PhoneSet");
-
     init_fsubr("defPhoneSet",make_phoneset,
  "(defPhoneSet PHONESETNAME FEATURES PHONEDEFS)\n\
   Define a new phoneset named PHONESETNAME.  Each phone is described with a\n\
