@@ -5,15 +5,19 @@
 /*                       Copyright (c) 1996,1997                         */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -162,16 +166,16 @@ LISP FT_English_Token_Utt(LISP utt)
 	}
 
 	// final word gets punctuation marking
-	if ((new_word != 0) && (t->f("punc") != "0"))
+	if ((new_word != 0) && (ffeature(t,"punc") != "0"))
 	{
-	    if ((t->f("punc") == ".") &&
+	    if ((ffeature(t,"punc") == ".") &&
 		(wagon_predict(t,eou_tree) == 0))
 	    {   // It wasn't a really punctuation mark
-		t->fset("punc","0");
+		t->set("punc","0");
 	    }
 	    else
 	    {
-		l = symbolexplode(strintern(t->f("punc").string()));
+		l = symbolexplode(strintern(ffeature(t,"punc").string()));
 		for (w=l; w != NIL; w=cdr(w))
 		{
 		    new_word = add_word(u,car(w));
@@ -200,7 +204,7 @@ static LISP word_it(EST_Item *token, const EST_String tok)
 
     if (user_token_to_word_func != NIL)            // check user's rules
 	return leval(cons(user_token_to_word_func,
-			  cons(siod_make_item(token),
+			  cons(siod(token),
 			       cons(tok_string,NIL))),NIL);
     else
 	return builtin_word_it(token,tok);
@@ -226,7 +230,7 @@ static LISP builtin_word_it(EST_Item *token, EST_String tok)
 	else
 	    return cons(strintern(tok),NIL);
     }
-    else if ((token_pos = token->f("token_pos")) == "ordinal")
+    else if ((token_pos = ffeature(token,"token_pos")) == "ordinal")
 	return say_num_as_ordinal(tok);
     else if (token_pos == "year")
 	return say_num_as_year(tok);
@@ -247,7 +251,7 @@ static LISP builtin_word_it(EST_Item *token, EST_String tok)
     }
     else if (tok.matches(RXapostropheS))
     {
-	return l_append(word_it(token,tok.at(0,tok.length()-2)),
+	return append(word_it(token,tok.at(0,tok.length()-2)),
 			cons(strintern("'s"),NIL));
     }
     else if (tok.matches(numpointnum))
@@ -256,8 +260,8 @@ static LISP builtin_word_it(EST_Item *token, EST_String tok)
 	LISP ap = NIL;
 	int i;
 	for (i=0; i < afterpoint.length(); i++)
-	     ap = l_append(say_num_as_words(afterpoint.at(i,1)),ap);
-	return l_append(say_num_as_words(tok.before(".")),
+	     ap = append(say_num_as_words(afterpoint.at(i,1)),ap);
+	return append(say_num_as_words(tok.before(".")),
 			cons(strintern("point"),reverse(ap)));
     }
     else if ((tok.matches(make_regex("[A-Z][A-Z]+"))) &&
@@ -275,53 +279,53 @@ static LISP builtin_word_it(EST_Item *token, EST_String tok)
     else if (only_punc(tok))
 	return stringexplode(tok);
     else if (tok.contains("-"))
-	return l_append(word_it(token,tok.before("-")),
+	return append(word_it(token,tok.before("-")),
 		      word_it(token,tok.after("-")));
     else if (tok.contains(".")) // internet address
     {
 	LISP a=NIL;
 	EST_String tok2 = tok;
 	for ( ; tok2.contains("."); tok2 = tok2.after("."))
-	    a = l_append(a,
-			 l_append(word_it(token,tok2.before(".")),
+	    a = append(a,
+			 append(word_it(token,tok2.before(".")),
 				  cons(strintern("dot"),NIL)));
-	a = l_append(a,word_it(token,tok2));
+	a = append(a,word_it(token,tok2));
 	return a;
     }
     else if (tok.contains("/")) 
-	return l_append(word_it(token,tok.before("/")),
+	return append(word_it(token,tok.before("/")),
 		      cons(strintern("slash"),
 			   word_it(token,tok.after("/"))));
     else if (tok.contains("&")) 
-	return l_append(word_it(token,tok.before("&")),
+	return append(word_it(token,tok.before("&")),
 		      cons(strintern("ampersand"),
 			   word_it(token,tok.after("&"))));
     else if (tok.contains("_")) 
-	return l_append(word_it(token,tok.before("_")),
+	return append(word_it(token,tok.before("_")),
 		      cons(strintern("underscore"),
 			   word_it(token,tok.after("_"))));
     else if (tok.contains("'"))
 	return word_it(token,tok.before("'")+tok.after("'"));
     else if (tok.contains("`"))
-	return l_append(word_it(token,tok.before("`")),
+	return append(word_it(token,tok.before("`")),
 			word_it(token,tok.after("`")));
     else if (tok.contains("\""))
-	return l_append(word_it(token,tok.before("\"")),
+	return append(word_it(token,tok.before("\"")),
 			word_it(token,tok.after("\"")));
     else if (tok.contains(","))
-	return l_append(word_it(token,tok.before(",")),
+	return append(word_it(token,tok.before(",")),
 			word_it(token,tok.after(",")));
     else if (tok.contains("("))
-	return l_append(word_it(token,tok.before("(")),
+	return append(word_it(token,tok.before("(")),
 			word_it(token,tok.after("(")));
     else if (tok.contains(")"))
-	return l_append(word_it(token,tok.before(")")),
+	return append(word_it(token,tok.before(")")),
 			word_it(token,tok.after(")")));
     else if (tok.matches(make_regex("^[^a-zA-Z].+"))) // incrementally remove 
-	return l_append(say_as_letters(tok.at(0,1)),// num/symbol from front
+	return append(say_as_letters(tok.at(0,1)),// num/symbol from front
 			word_it(token,tok.at(1,tok.length()-1)));
     else if (tok.matches(make_regex(".+[^a-zA-Z]$"))) // incrementally remove rear
-	return l_append(word_it(token,tok.at(0,tok.length()-1)),
+	return append(word_it(token,tok.at(0,tok.length()-1)),
 			say_as_letters(tok.at((int)tok.length()-1,1)));
     else  // could try harder
 	return say_as_letters(remove_punct(tok));
@@ -425,7 +429,7 @@ static LISP say_num_as_ordinal(const EST_String &num)
     
     // Now change the last word to the appropriate ordinal
     for (last=numwords; cdr(last) != NIL; last=cdr(last));
-    char *lastword = get_c_string(car(last));
+    const char *lastword = get_c_string(car(last));
     
     if (streq(lastword,"zero"))  
 	CAR(last) = strintern("zeroth");
@@ -493,6 +497,11 @@ static LISP say_num_as_year(const EST_String &num)
 
     if (num.length() > 4) 
 	return say_num_as_words(num);
+    else if (num.matches(make_regex("00")))
+    {
+	return cons(strintern("o"),
+		    cons(strintern("o"),NIL));
+    }
     else if (num.matches(make_regex("0[0-9]")))
     {
 	return cons(strintern("o"),
@@ -503,23 +512,23 @@ static LISP say_num_as_year(const EST_String &num)
     else if ((iword % 1000) < 10)
     {
 	if ((iword % 1000) == 0)
-	    return l_append(num_2_words(iword/1000),
+	    return append(num_2_words(iword/1000),
 			    cons(strintern("thousand"),NIL));
 	else
-	    return l_append(num_2_words(iword/1000),
+	    return append(num_2_words(iword/1000),
 			    cons(strintern("thousand"),
 				 cons(strintern("and"),
 				      num_2_words(iword%1000))));
     }
     else if ((iword % 100) == 0)
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			cons(strintern("hundred"),NIL));
     else if ((iword % 100) < 10)
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			cons(strintern("o"),
 			     num_2_words(iword%100)));
     else 
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			num_2_words(iword%100));
 }
     
@@ -527,7 +536,7 @@ static LISP num_2_words(int iword)
 {
     // Convert number of list of words
     int tens, units;
-    LISP s_tens;
+    LISP s_tens, lang_stype=NIL;
     
     if (iword < 0)
 	return cons(strintern("minus"),num_2_words(-iword));
@@ -580,15 +589,15 @@ static LISP num_2_words(int iword)
     }
     else if (iword < 1000)
     {
-	LISP lang_stype = ft_get_param("Language");
+	lang_stype = ft_get_param("Language");
 	if (streq("americanenglish",get_c_string(lang_stype)))
-	    return l_append(num_2_words(iword/100),
+	    return append(num_2_words(iword/100),
 			    cons(strintern("hundred"),
 				 (((iword % 100) != 0) ?
 				  num_2_words(iword % 100) :
 				  NIL)));
 	else
-	    return l_append(num_2_words(iword/100),
+	    return append(num_2_words(iword/100),
 			    cons(strintern("hundred"),
 				 (((iword % 100) != 0) ?
 				  cons(strintern("and"),
@@ -599,11 +608,11 @@ static LISP num_2_words(int iword)
     // We don't depend on this hack now.
     else if ((iword > 1910) &&
 	     (iword < 2000))   // hacky date condition
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			num_2_words(iword%100));
 #endif
     else if (iword < 1000000)
-	return l_append(num_2_words(iword/1000),
+	return append(num_2_words(iword/1000),
 			cons(strintern("thousand"),
 			     (((iword % 1000) != 0) ?
 			      ((((iword % 1000)/100) == 0) ?
@@ -611,7 +620,7 @@ static LISP num_2_words(int iword)
 			       num_2_words(iword % 1000)) :
 			      NIL)));
     else if (iword >= 1000000)
-	return l_append(num_2_words(iword/1000000),
+	return append(num_2_words(iword/1000000),
 			cons(strintern("million"),
 			     (((iword % 1000000) != 0) ?
 			      num_2_words(iword % 1000000) :
