@@ -5,15 +5,19 @@
 /*                       Copyright (c) 1996,1997                         */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,9 +41,6 @@
 #include "festival.h"
 #include "modules.h"
 
-static EST_String SylStructure("SylStructure");
-static EST_String Token("Token");
-static EST_String Phrase("Phrase");
 static EST_String stressname("stress");
 static EST_Val val_string0("0");
 static EST_Val val_string1("1");
@@ -47,14 +48,19 @@ static EST_Val val_int0(0);
 static EST_Val val_int1(1);
 static EST_Val default_val_float(0.0);
 
-static EST_Val nff_name(EST_Item *i)
+static EST_Val ff_addr(EST_Item *i)
 {
-    return EST_Val(i->name());
+    char a[1024];
+
+    // The address of the contents so that the same item from different views
+    // have the same address
+    sprintf(a,"%p",i->contents());  
+    return EST_Val(a);
 }
 
 static EST_Val ff_segment_duration(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("Segment");
+    EST_Item *n = as(s,"Segment");
     if (n == 0)
     {
 	cerr << "Asked for segment duration of item not in Segment relation."
@@ -62,44 +68,44 @@ static EST_Val ff_segment_duration(EST_Item *s)
 	festival_error();
     }
     if (prev(n) == 0)
-	return EST_Val(s->fF("end"));
+	return EST_Val(s->F("end", 0));
     else
-	return EST_Val(s->fF("end")-(prev(n)->fF("end")));
+	return EST_Val(s->F("end", 0)-(prev(n)->F("end",0)));
 }
 
 static EST_Val ff_syllable_duration(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("SylStructure");
+    EST_Item *n = as(s,"SylStructure");
     if (n == 0)
     {
 	cerr << "Asked for syllable duration of item not in SylStructure relation."
-	    << endl;
+	     << endl;
 	festival_error();
     }
     else
     {
-	EST_Item *fd = n->daughter1();
+	EST_Item *fd = daughter1(n);
 	EST_Item *ld = fd->last();
 
 	if (ld == 0)
 	    return val_int0;
-	EST_Item *ps = prev(fd->as_relation("Segment"));
+	EST_Item *ps = prev(as(fd,"Segment"));
 	if (ps == 0)
-	    return EST_Val(ld->fF("end"));
+	    return ld->F("end",0);
 	else
-	    return EST_Val(ld->fF("end")-ps->fF("end"));
+	    return EST_Val(ld->F("end",0)-ps->F("end",0));
     }
 
     // dummy for stupid VC++ compiler
     {
-      EST_Val junk;
-      return junk;
+	EST_Val junk;
+	return junk;
     }
 }
 
 static EST_Val ff_word_duration(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("SylStructure");
+    EST_Item *n = as(s,"SylStructure");
     if (n == 0)
     {
 	cerr << "Asked for word duration of item not in SylStructure relation."
@@ -108,16 +114,16 @@ static EST_Val ff_word_duration(EST_Item *s)
     }
     else
     {
-	EST_Item *fd = n->down()->down();
-	EST_Item *ld = n->down()->last()->down()->last();
+	EST_Item *fd = daughter1(daughter1(n));
+	EST_Item *ld = daughtern(daughtern(n));
 
 	if (ld == 0)
 	    return val_int0;
-	EST_Item *ps = prev(fd->as_relation("Segment"));
+	EST_Item *ps = prev(as(fd,"Segment"));
 	if (ps == 0)
-	    return EST_Val(ld->fF("end"));
+	    return ld->F("end",0);
 	else
-	    return EST_Val(ld->fF("end")-ps->fF("end"));
+	    return EST_Val(ld->F("end",0)-ps->F("end",0));
     }
     // dummy for stupid VC++ compiler
     {
@@ -128,17 +134,17 @@ static EST_Val ff_word_duration(EST_Item *s)
 
 static EST_Val ff_seg_start(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("Segment");
+    EST_Item *n = as(s,"Segment");
     if (prev(n) == 0)
 	return default_val_float;
     else
-	return EST_Val(prev(n)->fF("end"));
+	return prev(n)->F("end",0);
 }
 
 static EST_Val ff_syl_start(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("SylStructure");
-    if (daughters(n) == 0)
+    EST_Item *n = as(s,"SylStructure");
+    if (daughter1(n) == 0)
 	return default_val_float;
     else
 	return ff_seg_start(daughter1(n));
@@ -146,8 +152,8 @@ static EST_Val ff_syl_start(EST_Item *s)
 
 static EST_Val ff_word_start(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("SylStructure");
-    if (daughters(daughters(n)) == 0)
+    EST_Item *n = as(s,"SylStructure");
+    if (daughter1(daughter1(n)) == 0)
 	return default_val_float;
     else
 	return ff_seg_start(daughter1(daughter1(n)));
@@ -155,7 +161,7 @@ static EST_Val ff_word_start(EST_Item *s)
 
 static EST_Val ff_seg_end(EST_Item *s)
 {
-    return s->f("end");
+    return s->F("end",0);
 }
 
 static EST_Val ff_seg_mid(EST_Item *s)
@@ -165,7 +171,7 @@ static EST_Val ff_seg_mid(EST_Item *s)
 
 static EST_Val ff_syl_end(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("SylStructure");
+    EST_Item *n = as(s,"SylStructure");
     if (daughtern(n) == 0)
 	return default_val_float;
     else
@@ -174,7 +180,7 @@ static EST_Val ff_syl_end(EST_Item *s)
 
 static EST_Val ff_word_end(EST_Item *s)
 {
-    EST_Item *n = s->as_relation("SylStructure");
+    EST_Item *n = as(s,"SylStructure");
     if (daughtern(n) == 0)
 	return default_val_float;
     else
@@ -184,7 +190,7 @@ static EST_Val ff_word_end(EST_Item *s)
 static EST_Val ff_position_type(EST_Item *s)
 {
     /* Position of syllable in this word: initial, mod, final */
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
 
     if (nn == 0)  // its not really a syllable
 	return EST_Val("single");
@@ -204,10 +210,10 @@ static EST_Val ff_position_type(EST_Item *s)
 static EST_Val ff_word_break(EST_Item *w)
 {
     /* Break index of word */
-    EST_Item *nn = w->as_relation(Phrase);
-    static val4 = EST_Val(4);
-    static val3 = EST_Val(3);
-    static val2 = EST_Val(2);
+    EST_Item *nn = as(w,"Phrase");
+    static EST_Val val4 = EST_Val(4);
+    static EST_Val val3 = EST_Val(3);
+    static EST_Val val2 = EST_Val(2);
 
     if ((nn == 0) || (next(nn) != 0))
 	return val_int1;
@@ -234,7 +240,7 @@ static EST_Val ff_word_break(EST_Item *w)
 static EST_Val ff_syl_break(EST_Item *s)
 {
     // 0 internal syl end, 1 word end, 4 phrase end (ToBI 3 and 4)
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
 
     if (nn == 0)
 	return val_int1;  // no sylstructure so maybe its standalone
@@ -250,7 +256,7 @@ static EST_Val ff_old_syl_break(EST_Item *s)
 {
     // 0 internal syl end, 1 word end, 4 phrase end (ToBI 3 and 4)
     // 2's and threes are promoted to 4s
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     static EST_Val val4 = EST_Val(4);
 
     if (nn == 0)
@@ -272,7 +278,7 @@ static EST_Val ff_old_syl_break(EST_Item *s)
 static EST_Val ff_syl_accented(EST_Item *s)
 {
     // t if syllable is accented or not 
-    EST_Item *nn = s->as_relation("Intonation");
+    EST_Item *nn = as(s,"Intonation");
 
     if ((nn == 0) || (daughter1(nn) == 0))
 	return val_int0;
@@ -283,7 +289,7 @@ static EST_Val ff_syl_accented(EST_Item *s)
 static EST_Val ff_tobi_accent(EST_Item *s)
 {
     // First tobi accent related to syllable
-    EST_Item *nn = s->as_relation("Intonation");
+    EST_Item *nn = as(s,"Intonation");
     EST_Item *p;
 
     for (p=daughter1(nn); p; p=next(p))
@@ -295,13 +301,13 @@ static EST_Val ff_tobi_accent(EST_Item *s)
 static EST_Val ff_tobi_endtone(EST_Item *s)
 {
     // First tobi endtone (phrase accent or boundary tone)
-    EST_Item *nn = s->as_relation("Intonation");
+    EST_Item *nn = as(s,"Intonation");
     EST_Item *p;
 
     for (p=daughter1(nn); p; p=next(p))
     {
 	EST_String l = p->name();
-	if ((l.contains("%")) || (l == "H-") || (l == "L-"))
+	if ((l.contains("%")) || (l.contains("-")))
 	    return EST_Val(p->name());
     }
 
@@ -311,7 +317,7 @@ static EST_Val ff_tobi_endtone(EST_Item *s)
 static EST_Val ff_syl_accent(EST_Item *s)
 {
     // (first) accent or NONE on given syllable
-    EST_Item *nn = s->as_relation("Intonation");
+    EST_Item *nn = as(s,"Intonation");
 
     if (daughter2(nn))
 	return EST_Val("multi");
@@ -324,27 +330,27 @@ static EST_Val ff_syl_accent(EST_Item *s)
 static EST_Val ff_syl_numphones(EST_Item *s)
 {
     // Number of phones in syllable
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
 
-    return EST_Val(daughters(nn)->length());
+    return EST_Val(daughter1(nn)->length());
 }
 
 static EST_Val ff_word_numsyls(EST_Item *s)
 {
     // Number of syllable in word
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
 
-    return EST_Val(daughters(nn)->length());
+    return EST_Val(daughter1(nn)->length());
 }
 
 static EST_Val ff_syl_onsetsize(EST_Item *s)
 {
     // number of segments in the onset
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
     int size;
 
-    for (p=daughters(nn),size=0; p; p=next(p),size++)
+    for (p=daughter1(nn),size=0; p; p=next(p),size++)
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(size);
     
@@ -355,11 +361,11 @@ static EST_Val ff_syl_onsetsize(EST_Item *s)
 static EST_Val ff_syl_vowel(EST_Item *s)
 {
     // the vowel in the syllable
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
     int size;
 
-    for (p=daughters(nn),size=0; p; p=next(p),size++)
+    for (p=daughter1(nn),size=0; p; p=next(p),size++)
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(p->name());
 
@@ -370,11 +376,11 @@ static EST_Val ff_syl_vowel(EST_Item *s)
 static EST_Val ff_syl_codasize(EST_Item *s)
 {
     // number of segments in the coda
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
     int size;
 
-    for (p=daughters(nn)->last(),size=1; p; p=prev(p),size++)
+    for (p=daughter1(nn)->last(),size=1; p; p=prev(p),size++)
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(size);
     
@@ -384,35 +390,35 @@ static EST_Val ff_syl_codasize(EST_Item *s)
 static EST_Val ff_syl_pc_unvox(EST_Item *s)
 {
     // Returns percentage of syllable from start to first voiced phone
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p,*ps;
     float unvox,start = 0;
 
-    if (daughters(nn) == 0)
+    if (daughter1(nn) == 0)
 	return val_int0;  // no segments in syllable
-    else if ((ps = prev(daughters(nn)->as_relation("Segment"))) != 0)
-	start = ps->fF("end");
+    else if ((ps = prev(daughter1(nn),"Segment")) != 0)
+	start = ps->F("end",0);
     unvox = start;
 
-    for (p=daughters(nn); p != 0; p=next(p))
+    for (p=daughter1(nn); p != 0; p=next(p))
     {
 	if ((ph_is_vowel(p->name())) ||
 	    (ph_is_voiced(p->name())))
 	    break;
-	unvox = p->fF("end");
+	unvox = p->F("end",0);
     }
 
     return EST_Val((int)(((unvox-start)*100)/
-			 (daughtern(nn)->fF("end")-start)));
+			 (daughtern(nn)->F("end",0)-start)));
 }
 
 static EST_Val ff_syl_vowel_start(EST_Item *s)
 {
     // Returns start time of vowel in syllable (or start of syllable)
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
 
-    for (p=daughters(nn); p != 0; p=next(p))
+    for (p=daughter1(nn); p != 0; p=next(p))
     {
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(ff_seg_start(p));
@@ -424,7 +430,7 @@ static EST_Val ff_syl_vowel_start(EST_Item *s)
 static EST_Val ff_seg_onsetcoda(EST_Item *s)
 {
     // onset if seg in onset, coda otherwise (vowel is in coda)
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
 
     for (p=next(nn); p; p=next(p))
@@ -436,7 +442,7 @@ static EST_Val ff_seg_onsetcoda(EST_Item *s)
 static EST_Val ff_seg_onset_stop(EST_Item *s)
 {
     // 1 if onset of the syllable attached to this segment has a stop
-    EST_Item *nn = first(s->as_relation("SylStructure"));
+    EST_Item *nn = first(as(s,"SylStructure"));
 
     for ( ; nn ; nn=next(nn))
     {
@@ -451,7 +457,7 @@ static EST_Val ff_seg_onset_stop(EST_Item *s)
 static EST_Val ff_seg_coda_fric(EST_Item *s)
 {
     // 1 if coda of the syllable attached to this segment has a fricative
-    EST_Item *nn = last(s->as_relation("SylStructure"));
+    EST_Item *nn = last(as(s,"SylStructure"));
 
     for ( ; nn ; nn=prev(nn))
     {
@@ -466,7 +472,7 @@ static EST_Val ff_seg_coda_fric(EST_Item *s)
 static EST_Val ff_seg_pos_in_syl(EST_Item *s)
 {
     // position of segment in syllable
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
     int pos=0;
 
@@ -480,7 +486,7 @@ static EST_Val ff_seg_pos_in_syl(EST_Item *s)
 static EST_Val ff_seg_syl_initial(EST_Item *s)
 {
     // 1 if seg is syllable initial, 0 otherwise.
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
 
     if (prev(nn) == 0)
 	return val_string1;
@@ -491,7 +497,7 @@ static EST_Val ff_seg_syl_initial(EST_Item *s)
 static EST_Val ff_seg_syl_final(EST_Item *s)
 {
     // 1 if seg is syllable initial, 0 otherwise.
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
 
     if (next(nn) == 0)
 	return val_string1;
@@ -502,7 +508,7 @@ static EST_Val ff_seg_syl_final(EST_Item *s)
 static EST_Val ff_syl_pos_in_word(EST_Item *s)
 {
     // position of syllable in word
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
     int pos=0;
 
@@ -516,7 +522,7 @@ static EST_Val ff_syl_pos_in_word(EST_Item *s)
 static EST_Val ff_pos_in_phrase(EST_Item *s)
 {
     // position of word in phrase
-    EST_Item *nn = s->as_relation(Phrase);
+    EST_Item *nn = as(s,"Phrase");
     EST_Item *p;
     int pos=0;
 
@@ -531,7 +537,7 @@ static EST_Val ff_num_break(EST_Item *s)
 {
     // 1 if this word is at the end of a number group and followed by
     // a new number group
-    EST_Item *nn = s->as_relation(Token);
+    EST_Item *nn = as(s,"Token");
 
     if ((next(nn) == 0) &&
 	(parent(nn)->name().matches(RXdouble)) &&
@@ -543,16 +549,16 @@ static EST_Val ff_num_break(EST_Item *s)
 
 static EST_Val ff_words_out(EST_Item *s)
 {
-    return EST_Val(s->as_relation(Phrase)->length());
+    return EST_Val(as(s,"Phrase")->length());
 }
 
 static EST_Val ff_syl_midpitch(EST_Item *s)
 {
     // pitch of mid vowel in syllable
-    EST_Item *nn = s->as_relation("SylStructure");
+    EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
 
-    for (p=daughters(nn); p; p = next(p))
+    for (p=daughter1(nn); p; p = next(p))
     {
 	if (ph_is_vowel(p->name()))
 	    return ffeature(p,"R:Target.daughter1.f0");
@@ -600,7 +606,7 @@ static EST_Val ff_seg_pitch(EST_Item *s)
     for (lastt=t=u->relation("Target")->first_leaf(); 
 	 next_leaf(t) != 0; t=next_leaf(t))
     {
-	if (smid <= t->fF("pos"))
+	if (smid <= t->F("pos",0))
 	    break;
 	lastt=t;
     }
@@ -608,13 +614,13 @@ static EST_Val ff_seg_pitch(EST_Item *s)
     if (lastt == 0)
 	return EST_Val((float)0.0);
 
-    deltaf0 = t->fF("f0")-lastt->fF("f0");
-    deltatime = t->fF("pos") - lastt->fF("pos");
+    deltaf0 = t->F("f0",0)-lastt->F("f0",0);
+    deltatime = t->F("pos",0) - lastt->F("pos",0);
     if (deltatime <= 0)
-	spoint = lastt->fF("f0");
+	spoint = lastt->F("f0",0);
     else
-	spoint = lastt->fF("f0") + 
-	    (deltaf0*((smid-lastt->fF("pos"))/deltatime));
+	spoint = lastt->F("f0",0) + 
+	    (deltaf0*((smid-lastt->F("pos",0))/deltatime));
 
     if (spoint > 35)
 	return EST_Val(spoint);
@@ -625,12 +631,11 @@ static EST_Val ff_seg_pitch(EST_Item *s)
 static EST_Val ff_syl_in(EST_Item *s)
 {
     // Number of syllables to since last phrase break 
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     // The first syllable in the phrase
-    EST_Item *fsyl = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->first()->
-	    as_relation("SylStructure")->daughter1()->
-		as_relation("Syllable");
+    EST_Item *fsyl = 
+	as(daughter1(first(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	   "Syllable");
     EST_Item *p;
     int count;
 
@@ -643,12 +648,11 @@ static EST_Val ff_syl_in(EST_Item *s)
 static EST_Val ff_syl_out(EST_Item *s)
 {
     // Number of syllables since last phrase break 
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     // The last syllable in the phrase
-    EST_Item *lsyl = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->last()->
-	    as_relation("SylStructure")->daughtern()->
-		as_relation("Syllable");
+    EST_Item *lsyl = 
+	as(daughtern(last(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	   "Syllable");
     EST_Item *p;
     int count;
 
@@ -661,17 +665,16 @@ static EST_Val ff_syl_out(EST_Item *s)
 static EST_Val ff_ssyl_in(EST_Item *s)
 {
     // Number of stressed syllables since last phrase break 
-    EST_Item *nn = s->as_relation("Syllable");
-    EST_Item *fsyl = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->first()->
-	    as_relation("SylStructure")->daughter1()->
-		as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
+    EST_Item *fsyl = 
+	as(daughter1(first(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	   "Syllable");
     EST_Item *p;
     int count;
 
     if (nn == fsyl) return val_int0;
     for (count=0,p=prev(nn); (p != 0) && (p != fsyl); p = prev(p))
-	if (p->f(stressname) == 1)
+	if (p->F(stressname,0) == 1)
 	    count ++;
     return EST_Val(count);
 }
@@ -679,19 +682,18 @@ static EST_Val ff_ssyl_in(EST_Item *s)
 static EST_Val ff_ssyl_out(EST_Item *s)
 {
     // Number of stressed syllables to next phrase break 
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     // The last syllable in the phrase
-    EST_Item *lsyl = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->last()->
-	    as_relation("SylStructure")->daughtern()->
-		as_relation("Syllable");
+    EST_Item *lsyl = 
+	as(daughtern(last(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	   "Syllable");
     EST_Item *p;
     int count;
 
     if (nn == lsyl) return val_int0;
     for (count=0,p=next(nn); (p != 0); p=next(p))
     {
-	if (p->f(stressname) == 1)
+	if (p->F(stressname,0) == 1)
 	    count ++;
 	if (p == lsyl) break;
     }
@@ -701,12 +703,11 @@ static EST_Val ff_ssyl_out(EST_Item *s)
 static EST_Val ff_asyl_in(EST_Item *s)
 {
     // Number of accented syllables since last phrase break 
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     // The first syllable in the phrase
-    EST_Item *fsyl = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->first()->
-	    as_relation("SylStructure")->daughter1()->
-		as_relation("Syllable");
+    EST_Item *fsyl = 
+	as(daughter1(first(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	   "Syllable");
     EST_Item *p;
     int count;
 
@@ -720,12 +721,11 @@ static EST_Val ff_asyl_in(EST_Item *s)
 static EST_Val ff_asyl_out(EST_Item *s)
 {
     // Number of accented syllables to next phrase break 
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     // The last syllable in the phrase
-    EST_Item *lsyl = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->last()->
-	    as_relation("SylStructure")->daughtern()->
-		as_relation("Syllable");
+    EST_Item *lsyl = 
+	as(daughtern(last(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	   "Syllable");
     EST_Item *p;
     int count;
 
@@ -742,7 +742,7 @@ static EST_Val ff_asyl_out(EST_Item *s)
 static EST_Val ff_last_accent(EST_Item *s)
 {
     // Number of syllables since last accented syllable
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     EST_Item *p;
     int count;
 
@@ -756,7 +756,7 @@ static EST_Val ff_last_accent(EST_Item *s)
 static EST_Val ff_next_accent(EST_Item *s)
 {
     // Number of syllables to next accented syllable
-    EST_Item *nn = s->as_relation("Syllable");
+    EST_Item *nn = as(s,"Syllable");
     EST_Item *p;
     int count;
 
@@ -770,8 +770,7 @@ static EST_Val ff_next_accent(EST_Item *s)
 static EST_Val ff_sub_phrases(EST_Item *s)
 {
     // Number of non-major phrase breaks since last major phrase break
-    EST_Item *nn = s->as_relation("SylStructure")->parent()->
-	as_relation("Phrase")->parent();
+    EST_Item *nn = parent(parent(s,"SylStructure"),"Phrase");
     EST_Item *p;
     int count;
 
@@ -818,15 +817,15 @@ void festival_ff_init(void)
     "Segment.segment_end\n\
   The end time of the given segment.");
     festival_def_nff("syllable_end","Syllable",ff_syl_end,
-    "Word.word_end\n\
+    "Syllable.syllable_end\n\
   The end time of the given syllable.");
     festival_def_nff("word_end","Word",ff_word_end,
     "Word.word_end\n\
   The end time of the given word.");
-    festival_def_nff("name","ANY",nff_name,
-    "ANY.name\n\
-  The name of the stream item.  This may be applied to stream items of\n\
-  any stream type.");
+    festival_def_nff("addr","ANY",ff_addr,
+    "ANY.addr\n\
+  Returned by popular demand, returns the address of given item that\n\
+  is guaranteed unique for this session.");
 
     festival_def_nff("accented","Syllable",ff_syl_accented,
     "Syllable.accented\n\
@@ -923,7 +922,7 @@ void festival_ff_init(void)
   including this one.  Accentedness is as defined by the syl_accented\n\
   feature.");
     festival_def_nff("asyl_out","Syllable",ff_asyl_out,
-    "Syllable.asyl_in\n\
+    "Syllable.asyl_out\n\
   Returns number of accented syllables to the next phrase break, not\n\
   including this one.  Accentedness is as defined by the syl_accented\n\
   feature.");
@@ -993,4 +992,3 @@ void festival_ff_init(void)
   poly-syllabic words.");
 
 }
-    
